package lcsb.mapviewer.services.search.comment;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import com.google.gson.Gson;

import lcsb.mapviewer.common.Pair;
import lcsb.mapviewer.common.exception.InvalidArgumentException;
import lcsb.mapviewer.model.map.Comment;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.services.overlay.IconManager;
import lcsb.mapviewer.services.search.SearchResultFactory;
import lcsb.mapviewer.services.search.data.ElementIdentifier;
import lcsb.mapviewer.services.search.data.ElementIdentifier.ElementIdentifierType;

/**
 * Factory class for {@link FullCommentView} class.
 * 
 * @author Piotr Gawron
 * 
 */
public class FullCommentViewFactory extends SearchResultFactory<List<Comment>, FullCommentView> {

	/**
	 * Default class logger.
	 */
	@SuppressWarnings("unused")
	private static Logger logger = Logger.getLogger(FullCommentViewFactory.class);

	@Override
	public FullCommentView create(List<Comment> list) {
		Point2D point = new Point2D.Double(0, 0);
		Comment comment = list.get(0);
		Class<?> type;
		FullCommentView result;
		if (comment.getTableName() != null) {
			result = new FullCommentView(comment.getTableId(), comment.getSubmodelData().getId());
			if (comment.getTableName().getName().contains("Reaction")) {
				type = Reaction.class;
			} else {
				type = Element.class;
			}
		} else {
			point.setLocation(comment.getCoordinates().getX(), comment.getCoordinates().getY());
			result = new FullCommentView(point, comment.getSubmodelData().getId());
			type = Point2D.class;
		}
		result.setType(type);

		for (int i = 0; i < list.size(); i++) {
			Pair<String, String> pair = new Pair<String, String>("" + list.get(i).getId(), list.get(i).getContent());
			result.getComments().add(pair);
		}
		return result;
	}

	@Override
	public String createGson(FullCommentView object) {
		return new Gson().toJson(object);
	}

	@Override
	public List<ElementIdentifier> searchResultToElementIdentifier(FullCommentView comment, Model model) {
		List<ElementIdentifier> result = new ArrayList<>();
		if (Reaction.class.equals(comment.getType())) {
			result
					.add(new ElementIdentifier(comment.getIdObject(), comment.getModelId(), ElementIdentifierType.REACTION, IconManager.getInstance().getCommentIcon()));
		} else if (Element.class.equals(comment.getType())) {
			result.add(new ElementIdentifier(comment.getIdObject(), comment.getModelId(), ElementIdentifierType.ALIAS, IconManager.getInstance().getCommentIcon()));
		} else if (Point2D.class.equals(comment.getType())) {
			result.add(new ElementIdentifier(comment.getIdObject(), comment.getModelId(), ElementIdentifierType.POINT, IconManager.getInstance().getCommentIcon()));
		} else {
			throw new InvalidArgumentException("Unknown comment type: " + comment.getType());
		}
		return result;
	}

}
