package lcsb.mapviewer.services.search.db;

import java.io.Serializable;

import lcsb.mapviewer.services.view.AnnotationView;

/**
 * Class representing gene that is included in the target. One target can
 * contain more than one gene (however, usually it's one).
 * 
 * @author Piotr Gawron
 * 
 */
public class GeneRow implements Serializable {
	/**
	 * 
	 */
	private static final long	serialVersionUID = 1L;

	/**
	 * Annotation of the gene (should be hgnc, but any other is also possible).
	 */
	private AnnotationView		annotation;

	/**
	 * Can the gene be selected by client.
	 */
	private Boolean						selectable			 = false;

	/**
	 * Is the gene selected by client.
	 */
	private Boolean						selected				 = false;

	/**
	 * Is the gene visible to the client.
	 */
	private Boolean						visible					 = false;

	/**
	 * Icon associated with the gene.
	 */
	private String						icon;

	/**
	 * @return the selectable
	 * @see #selectable
	 */
	public Boolean getSelectable() {
		return selectable;
	}

	/**
	 * @param selectable
	 *          the selectable to set
	 * @see #selectable
	 */
	public void setSelectable(Boolean selectable) {
		this.selectable = selectable;
	}

	/**
	 * @return the selected
	 * @see #selected
	 */
	public Boolean getSelected() {
		return selected;
	}

	/**
	 * @param selected
	 *          the selected to set
	 * @see #selected
	 */
	public void setSelected(Boolean selected) {
		this.selected = selected;
	}

	/**
	 * @return the visible
	 * @see #visible
	 */
	public Boolean getVisible() {
		return visible;
	}

	/**
	 * @param visible
	 *          the visible to set
	 * @see #visible
	 */
	public void setVisible(Boolean visible) {
		this.visible = visible;
	}

	/**
	 * @return the icon
	 * @see #icon
	 */
	public String getIcon() {
		return icon;
	}

	/**
	 * @param icon
	 *          the icon to set
	 * @see #icon
	 */
	public void setIcon(String icon) {
		this.icon = icon;
	}

	/**
	 * @return the annotation
	 * @see #annotation
	 */
	public AnnotationView getAnnotation() {
		return annotation;
	}

	/**
	 * @param annotation
	 *          the annotation to set
	 * @see #annotation
	 */
	public void setAnnotation(AnnotationView annotation) {
		this.annotation = annotation;
	}

}
