package lcsb.mapviewer.services.search.layout;

import org.apache.log4j.Logger;

import com.google.gson.Gson;

import lcsb.mapviewer.common.Pair;
import lcsb.mapviewer.model.map.layout.ColorSchema;
import lcsb.mapviewer.model.map.layout.GeneVariationColorSchema;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.services.search.ElementViewFactory;
import lcsb.mapviewer.services.utils.data.ColorSchemaType;

/**
 * Factory class for {@link FullLayoutAliasView} class.
 * 
 * @author Piotr Gawron
 * 
 */
public class FullLayoutAliasViewFactory extends ElementViewFactory<Pair<Element, ColorSchema>, FullLayoutAliasView> {
	/**
	 * Default class logger.
	 */
	@SuppressWarnings("unused")
	private static Logger logger = Logger.getLogger(FullLayoutAliasViewFactory.class);

	@Override
	public FullLayoutAliasView create(Pair<Element, ColorSchema> pair) {
		ColorSchema schema = pair.getRight();

		FullLayoutAliasView result = new FullLayoutAliasView(pair.getLeft());
		result.setColor(schema.getColor());
		result.setValue(schema.getValue());
		result.setDescription(schema.getDescription());
		if (schema instanceof GeneVariationColorSchema) {
			result.setGeneVariations(((GeneVariationColorSchema) schema).getGeneVariations());
			result.setType(ColorSchemaType.GENETIC_VARIANT);
		} else {
			result.setType(ColorSchemaType.GENERIC);
		}
		return result;
	}

	@Override
	public String createGson(FullLayoutAliasView object) {
		return new Gson().toJson(object);
	}

}
