package lcsb.mapviewer.services.view;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import lcsb.mapviewer.model.map.graph.DataMining;

/**
 * This class is a view of {@link DataMining data mining} element.
 * 
 * @author Piotr Gawron
 * 
 */
public class DataMiningView extends AbstractView<DataMining> implements Serializable {

	/**
	 * 
	 */
	private static final long		 serialVersionUID	 = 1L;

	/**
	 * List of elements that shuold be added.
	 */
	private List<AnnotationView> suggestedElements = new ArrayList<AnnotationView>();

	/**
	 * References that support this entry.
	 */
	private List<AnnotationView> references				 = new ArrayList<AnnotationView>();

	/**
	 * Source of the analysis (url).
	 */
	private String							 source						 = "";

	/**
	 * Type of the analysis (common name).
	 */
	private String							 type							 = "";

	/**
	 * Description that explains the entry.
	 */
	private String							 description			 = "";

	/**
	 * Default constructor. Should be used only for deserialization.
	 */
	protected DataMiningView() {
	}

	/**
	 * Default constructor that creates view for the object.
	 * 
	 * @param object
	 *          original data mining data
	 */
	protected DataMiningView(DataMining object) {
		super(object);
	}

	/**
	 * Adds element into {@link #suggestedElements}.
	 * 
	 * @param suggestedElement
	 *          object to add
	 */
	public void addSuggestedElement(AnnotationView suggestedElement) {
		suggestedElements.add(suggestedElement);
	}

	/**
	 * @return the suggestedElements
	 * @see #suggestedElements
	 */
	public List<AnnotationView> getSuggestedElements() {
		return suggestedElements;
	}

	/**
	 * @param suggestedElements
	 *          the suggestedElements to set
	 * @see #suggestedElements
	 */
	public void setSuggestedElements(List<AnnotationView> suggestedElements) {
		this.suggestedElements = suggestedElements;
	}

	/**
	 * @return the references
	 * @see #references
	 */
	public List<AnnotationView> getReferences() {
		return references;
	}

	/**
	 * @param references
	 *          the references to set
	 * @see #references
	 */
	public void setReferences(List<AnnotationView> references) {
		this.references = references;
	}

	/**
	 * @return the source
	 * @see #source
	 */
	public String getSource() {
		return source;
	}

	/**
	 * @param source
	 *          the source to set
	 * @see #source
	 */
	public void setSource(String source) {
		this.source = source;
	}

	/**
	 * @return the type
	 * @see #type
	 */
	public String getType() {
		return type;
	}

	/**
	 * @param type
	 *          the type to set
	 * @see #type
	 */
	public void setType(String type) {
		this.type = type;
	}

	/**
	 * @return the description
	 * @see #description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * @param description
	 *          the description to set
	 * @see #description
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * Adds reference that support this entry into {@link #references}.
	 * 
	 * @param reference
	 *          object to add
	 */
	public void addReference(AnnotationView reference) {
		references.add(reference);

	}

}
