package lcsb.mapviewer.services.view;

import lcsb.mapviewer.model.map.MiriamData;
import lcsb.mapviewer.model.map.graph.DataMining;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import lcsb.mapviewer.common.exception.NotImplementedException;

import com.google.gson.Gson;

/**
 * Factory class for {@link DataMiningView} class.
 * 
 * @author Piotr Gawron
 * 
 */
public class DataMiningViewFactory extends AbstractViewFactory<DataMining, DataMiningView> {
	/**
	 * Default class logger.
	 */
	@SuppressWarnings("unused")
	private static Logger					logger	= Logger.getLogger(DataMiningViewFactory.class);

	/**
	 * Factory object for {@link AnnotationView} elements.
	 */
	@Autowired
	private AnnotationViewFactory	annotationViewFactory;

	@Override
	public DataMiningView create(DataMining dataMining) {
		DataMiningView view = new DataMiningView(dataMining);
		if (dataMining == null) {
			return view;
		}
		view.setSource(dataMining.getType().getSource());

		for (MiriamData md : dataMining.getSuggestedConnections()) {
			AnnotationView suggestedElement = annotationViewFactory.create(md);
			view.addSuggestedElement(suggestedElement);
		}
		view.setDescription(dataMining.getDescription());
		for (MiriamData reference : dataMining.getReferences()) {
			AnnotationView annotation = annotationViewFactory.create(reference);
			view.addReference(annotation);
		}
		return view;
	}


	@Override
	public String createGson(DataMiningView object) {
		return new Gson().toJson(object);
	}

	@Override
	public DataMining viewToObject(DataMiningView view) {
		throw new NotImplementedException();
	}
}
