package lcsb.mapviewer.services.view;

import lcsb.mapviewer.common.exception.NotImplementedException;
import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.model.map.layout.Layout;

import org.apache.log4j.Logger;

import com.google.gson.Gson;

/**
 * Factory class for {@link LayoutView} class.
 * 
 * @author Piotr Gawron
 * 
 */
public class LayoutViewFactory extends AbstractViewFactory<Layout, LayoutView> {
	/**
	 * Default class logger.
	 */
	@SuppressWarnings("unused")
	private static Logger logger = Logger.getLogger(LayoutViewFactory.class);

	@Override
	public LayoutView create(Layout layout) {
		LayoutView result = new LayoutView(layout);
		if (layout == null) {
			return result;
		}
		if (layout.getModel() != null) {
			result.setModelId(layout.getModel().getId());
		}
		result.setName(layout.getTitle());
		Project project = getProjectForLayout(layout);
		if (project != null && project.getDirectory() != null) {
			result.setDirectory(project.getDirectory() + "/" + layout.getDirectory());
		} else {
			result.setDirectory(layout.getDirectory());
		}
		if (layout.getCreator() != null) {
			result.setCreator(layout.getCreator().getName() + " " + layout.getCreator().getSurname());
		} else {
			result.setCreator("");
		}
		result.setStatus(layout.getStatus().getCommonName());
		result.setProgress(String.format("%1$,.2f", layout.getProgress()));

		result.setDescription(layout.getDescription());
		if (layout.getInputData() != null) {
			result.setInputDataAvailable("true");
		} else {
			result.setInputDataAvailable("false");
		}

		return result;
	}

	/**
	 * Returns projects associated with model. It might be tricky, because some
	 * layouts are connected to models that are not directly attached to project
	 * (they are attached by parent or grand-parent).
	 * 
	 * @param layout
	 *          layout for which project will be returned
	 * @return {@link Project} to which {@link Layout layout} belongs to
	 */
	protected Project getProjectForLayout(Layout layout) {
		if (layout.getModel() != null) {
			if (layout.getModel().getProject() != null) {
				return layout.getModel().getProject();
			}
		}
		if (layout.getParentLayout() != null) {
			return getProjectForLayout(layout.getParentLayout());
		}
		return null;
	}

	@Override
	public String createGson(LayoutView object) {
		return new Gson().toJson(object);
	}

	@Override
	public Layout viewToObject(LayoutView view) {
		throw new NotImplementedException();
	}
}
