package lcsb.mapviewer.services.view;

import java.awt.geom.Point2D;

import lcsb.mapviewer.common.exception.NotImplementedException;
import lcsb.mapviewer.model.map.OverviewImageLink;
import lcsb.mapviewer.model.map.OverviewLink;
import lcsb.mapviewer.model.map.OverviewModelLink;
import lcsb.mapviewer.model.map.OverviewSearchLink;
import lcsb.mapviewer.services.utils.gmap.CoordinationConverter;

import org.apache.log4j.Logger;
import org.springframework.transaction.annotation.Transactional;

import com.google.gson.Gson;

/**
 * Factory class for {@link OverviewLink} class and subclasses.
 * 
 * @author Piotr Gawron
 * 
 */
@Transactional(value = "txManager")
public class OverviewLinkViewFactory extends AbstractViewFactory<OverviewLink, OverviewLinkView> {
	/**
	 * Default class logger.
	 */
	private static Logger	logger	= Logger.getLogger(OverviewLinkViewFactory.class);

	@Override
	public OverviewLinkView create(OverviewLink object) {
		OverviewLinkView result = new OverviewLinkView(object);
		if (object == null) {
			logger.warn("Empty image...");
			return result;
		}
		result.setPolygon(object.getPolygon());

		if (object instanceof OverviewModelLink) {
			result.setModelLinkId(((OverviewModelLink) object).getLinkedModel().getId());
			CoordinationConverter cc = new CoordinationConverter(((OverviewModelLink) object).getLinkedModel());
			result.setLatLng(cc.toLatLng(new Point2D.Double(((OverviewModelLink) object).getxCoord(), ((OverviewModelLink) object).getyCoord())));
			result.setZoomLevel(((OverviewModelLink) object).getZoomLevel());

		} else if (object instanceof OverviewImageLink) {
			result.setImageLinkId(((OverviewImageLink) object).getLinkedOverviewImage().getId());
		} else if (object instanceof OverviewSearchLink) {
			result.setQuery(((OverviewSearchLink) object).getQuery());
		} else {
			throw new NotImplementedException("Not implemented behaviour for class: " + object.getClass());
		}
		result.setType(object.getClass().getSimpleName());

		return result;
	}

	@Override
	public String createGson(OverviewLinkView object) {
		return new Gson().toJson(object);
	}

	@Override
	public OverviewLink viewToObject(OverviewLinkView view) {
		throw new NotImplementedException();
	}
}
