package lcsb.mapviewer.services.view;

import java.io.FileNotFoundException;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import com.google.gson.Gson;

import lcsb.mapviewer.annotation.cache.BigFileCache;
import lcsb.mapviewer.common.exception.NotImplementedException;
import lcsb.mapviewer.model.map.layout.ReferenceGenomeGeneMapping;

/**
 * Factory class for {@link AnnotationView} class.
 * 
 * @author Piotr Gawron
 * 
 */
public class ReferenceGenomeGeneMappingViewFactory extends AbstractViewFactory<ReferenceGenomeGeneMapping, ReferenceGenomeGeneMappingView> {

	/**
	 * Default class logger.
	 */
	@SuppressWarnings("unused")
	private static Logger	logger = Logger.getLogger(ReferenceGenomeGeneMappingViewFactory.class);

	/**
	 * Service that allows to access big files (reference genome is stored as a
	 * big file on the server).
	 */
	@Autowired
	private BigFileCache	bigFileCache;

	@Override
	public ReferenceGenomeGeneMappingView create(ReferenceGenomeGeneMapping object) {
		ReferenceGenomeGeneMappingView result = new ReferenceGenomeGeneMappingView();
		result.setDownloadProgress(object.getDownloadProgress());
		try {
			result.setLocalUrl("../" + bigFileCache.getLocalPathForFile(object.getSourceUrl()));
		} catch (FileNotFoundException e) {
			result.setLocalUrl(null);
		}
		result.setSourceUrl(object.getSourceUrl());
		result.setName(object.getName());
		return result;
	}

	@Override
	public String createGson(ReferenceGenomeGeneMappingView object) {
		return new Gson().toJson(object);
	}

	@Override
	public ReferenceGenomeGeneMapping viewToObject(ReferenceGenomeGeneMappingView view) {
		throw new NotImplementedException();
	}

}
