package lcsb.mapviewer.services.view;

import java.io.FileNotFoundException;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import com.google.gson.Gson;

import lcsb.mapviewer.annotation.cache.BigFileCache;
import lcsb.mapviewer.common.exception.NotImplementedException;
import lcsb.mapviewer.model.map.MiriamData;
import lcsb.mapviewer.model.map.layout.ReferenceGenome;

/**
 * Factory class for {@link AnnotationView} class.
 * 
 * @author Piotr Gawron
 * 
 */
public class ReferenceGenomeViewFactory extends AbstractViewFactory<ReferenceGenome, ReferenceGenomeView> {

	/**
	 * Default class logger.
	 */
	private static Logger													logger = Logger.getLogger(ReferenceGenomeViewFactory.class);

	/**
	 * Access point for big files.
	 */
	@Autowired
	private BigFileCache													bigFileCache;

	/**
	 * Factory that changes {@link MiriamData} into readable annotations.
	 */
	@Autowired
	private AnnotationViewFactory									annotationViewFactory;

	/**
	 * Factory that creates readable gene mappings.
	 */
	@Autowired
	private ReferenceGenomeGeneMappingViewFactory	referenceGenomeGeneMappingViewFactory;

	@Override
	public ReferenceGenomeView create(ReferenceGenome object) {
		ReferenceGenomeView result = new ReferenceGenomeView();
		result.setDownloadProgress(object.getDownloadProgress());
		result.setGeneMapping(referenceGenomeGeneMappingViewFactory.createList(object.getGeneMapping()));
		try {
			result.setLocalUrl("../" + bigFileCache.getLocalPathForFile(object.getSourceUrl()));
		} catch (FileNotFoundException e) {
			logger.warn("Cannot find local file", e);
			result.setLocalUrl(null);
		}
		result.setOrganism(annotationViewFactory.create(object.getOrganism()));
		result.setSourceUrl(object.getSourceUrl());
		result.setType(object.getType());
		result.setVersion(object.getVersion());
		return result;
	}

	@Override
	public String createGson(ReferenceGenomeView object) {
		return new Gson().toJson(object);
	}

	@Override
	public ReferenceGenome viewToObject(ReferenceGenomeView view) {
		throw new NotImplementedException();
	}

}
