package lcsb.mapviewer.services.impl;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.fail;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.annotation.Rollback;

import lcsb.mapviewer.annotation.services.PubmedParser;
import lcsb.mapviewer.common.Pair;
import lcsb.mapviewer.common.exception.InvalidArgumentException;
import lcsb.mapviewer.converter.graphics.MapGenerator;
import lcsb.mapviewer.converter.graphics.MapGenerator.MapGeneratorParams;
import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.model.map.layout.Layout;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.services.ServiceTestFunctions;
import lcsb.mapviewer.services.interfaces.IModelService;
import lcsb.mapviewer.services.search.data.LightAliasView;
import lcsb.mapviewer.services.view.ProjectView;

@Rollback(true)
public class ModelServiceTest extends ServiceTestFunctions {
	static Logger	logger = Logger.getLogger(ModelServiceTest.class);
	@Autowired
	IModelService	modelService;

	@Autowired
	PubmedParser	backend;

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testRemoveModel() throws Exception {
		try {
			String directory = "test_directory";
			String projectId = "Some_id";
			Project project = new Project();
			project.setProjectId(projectId);
			projectDao.add(project);

			Model model = getModelForFile("testFiles/sample.xml", false);
			model.addLayout(new Layout("test", directory, false));

			project.addModel(model);
			projectDao.update(project);

			MapGenerator generator = new MapGenerator();
			MapGeneratorParams params = generator.new MapGeneratorParams().model(model).directory(directory);
			generator.generateMapImages(params);

			ProjectView row = projectService.getProjectViewByProjectId(project.getProjectId());
			projectService.removeProject(row, null, false);

			File file = new File(directory);
			assertFalse(file.exists());

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}

	}

	@Test
	public void testGetAliasByIds() throws Exception {
		try {
			Model model = getModelForFile("testFiles/sample.xml", false);
			int counter = 0;
			Element alias0 = null;
			for (Element alias : model.getElements()) {
				if (counter == 0) {
					alias0 = alias;
				}
				alias.setId(counter++);
			}
			List<Pair<Integer, Integer>> identifiers = new ArrayList<>();
			identifiers.add(new Pair<Integer, Integer>(0, 0));
			List<LightAliasView> list = modelService.getLightAliasesByIds(model, identifiers);
			assertEquals(1, list.size());
			assertEquals(alias0.getX(), list.get(0).getBounds().getX(), EPSILON);
			assertEquals(alias0.getY(), list.get(0).getBounds().getY(), EPSILON);
			assertEquals(alias0.getWidth(), list.get(0).getBounds().getWidth(), EPSILON);
			assertEquals(alias0.getHeight(), list.get(0).getBounds().getHeight(), EPSILON);
			identifiers.add(new Pair<Integer, Integer>(0, 5));
			list = modelService.getLightAliasesByIds(model, identifiers);
			assertEquals(2, list.size());
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}

	}

	@Test
	public void testGetAliasByInvalidIds() throws Exception {
		try {
			Model model = getModelForFile("testFiles/db_problem_model.xml", false);
			List<Pair<Integer, Integer>> identifiers = new ArrayList<>();
			// check with wrong alias id
			identifiers.add(new Pair<Integer, Integer>(0, 1));
			try {
				modelService.getLightAliasesByIds(model, identifiers);
				fail(InvalidArgumentException.class + " expected");
			} catch (InvalidArgumentException e) {

			}
			identifiers = new ArrayList<>();
			// check with wrong model id
			identifiers.add(new Pair<Integer, Integer>(1, 0));
			try {
				modelService.getLightAliasesByIds(model, identifiers);
				fail(InvalidArgumentException.class + " expected");
			} catch (InvalidArgumentException e) {

			}
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}

	}

	@Test
	public void testAddModel() throws Exception {
		try {
			String projectId = "Some_id";
			Project project = new Project();
			project.setProjectId(projectId);
			projectDao.add(project);

			Model model = getModelForFile("testFiles/db_problem_model.xml", false);

			project.addModel(model);
			projectDao.update(project);

			ProjectView row = projectService.getProjectViewByProjectId(project.getProjectId());
			projectService.removeProject(row, null, false);

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}

	}

}
