package lcsb.mapviewer.services.search.db.drug;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import lcsb.mapviewer.annotation.data.Drug;
import lcsb.mapviewer.annotation.data.Target;
import lcsb.mapviewer.annotation.data.TargetType;
import lcsb.mapviewer.model.map.MiriamData;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.services.ServiceTestFunctions;
import lcsb.mapviewer.services.overlay.IconManager;
import lcsb.mapviewer.services.search.data.ElementIdentifier;
import lcsb.mapviewer.services.search.db.TargetDetails;
import lcsb.mapviewer.services.search.db.TargetView;
import lcsb.mapviewer.services.search.db.TargetViewFactory;
import lcsb.mapviewer.services.view.AnnotationView;
import lcsb.mapviewer.services.view.AnnotationViewFactory;

public class DrugViewFactoryTest extends ServiceTestFunctions {

	@Autowired
	TargetViewFactory			targetViewFactory;

	Logger								logger = Logger.getLogger(DrugViewFactoryTest.class);

	@Autowired
	AnnotationViewFactory	avFactory;

	@Autowired
	DrugViewFactory				dvf;

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testGeneToOverlayList() throws Exception {
		try {
			Model model = getModelForFile("testFiles/drug_target.xml", true);
			model.setTileSize(256);
			List<Model> models = new ArrayList<>();
			models.add(model);

			AnnotationView annotation = avFactory.create(new MiriamData());
			annotation.setName("M");

			List<ElementIdentifier> overlays = dvf.geneToOverlayList(annotation, IconManager.getInstance().getEmpyIcon(), models);
			assertEquals(1, overlays.size());

			annotation.setName("GRIN2B");
			overlays = dvf.geneToOverlayList(annotation, IconManager.getInstance().getEmpyIcon(), models);
			assertEquals(3, overlays.size());

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testComplexToOverlayList() throws Exception {
		try {
			Drug drug = chemblParser.findDrug("AMANTADINE");
			Target exampleTarget = null;
			assertTrue(drug.getTargets().size() > 0);
			for (Target target : drug.getTargets()) {
				if (target.getName().equals("Glutamate [NMDA] receptor")) {
					target.setType(TargetType.COMPLEX_PROTEIN);
					exampleTarget = target;
				}
				logger.debug(target);
			}
			assertNotNull(exampleTarget);
			Model model = getModelForFile("testFiles/drug_target.xml", true);
			model.setTileSize(256);
			List<Model> models = new ArrayList<>();
			models.add(model);
			TargetView view = targetViewFactory.create(exampleTarget);
			view.setIcon(IconManager.getInstance().getEmpyIcon());

			List<ElementIdentifier> overlays = dvf.complexToOverlayList(view, models);
			assertEquals(2, overlays.size());

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testgetElementInformationForResult() throws Exception {
		try {
			Drug drug = chemblParser.findDrug("AMANTADINE");
			Model model = getModelForFile("testFiles/drug_target.xml", false);
			Element alias = model.getElementByElementId("sa9");
			alias.setName("M");
			alias.setId(12);
			model.removeElement(alias);
			model.addElement(alias);
			ElementIdentifier ei = new ElementIdentifier(alias, null);
			model.setTileSize(256);
			List<Model> models = new ArrayList<>();
			models.add(model);
			DrugView view = dvf.create(drug);
			TargetDetails result = dvf.getElementInformationForResult(view.getTargetRows(), ei, model);
			assertNotNull(result);
			assertFalse(result.getDatabaseSource().equals(result.getReferences().get(0)));
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testCreateEmpty() throws Exception {
		try {
			Object object = dvf.create(null);
			assertNotNull(object);
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testCreateGson() throws Exception {
		try {
			DrugView object = dvf.create(null);
			assertNotNull(dvf.createGson(object));
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

}
