package lcsb.mapviewer.services.search.db.mirna;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import org.apache.log4j.Logger;

import lcsb.mapviewer.annotation.data.MiRNA;
import lcsb.mapviewer.common.comparator.StringComparator;
import lcsb.mapviewer.services.search.ISearchResultView;
import lcsb.mapviewer.services.search.db.TargetView;
import lcsb.mapviewer.services.view.AbstractView;

/**
 * Data of the MiRNA from external database to be visualized in the client side.
 * 
 * @author Ayan Rota
 * 
 */
public class MiRNAView extends AbstractView<MiRNA> implements Serializable, ISearchResultView {
	/**
	 * 
	 */
	private static final long	serialVersionUID = 1L;

	/**
	 * Default class logger.
	 */
	@SuppressWarnings("unused")
	private static Logger			logger					 = Logger.getLogger(MiRNAView.class);

	/**
	 * Name of the miRNA.
	 */
	private String						name;

	/**
	 * Is the MiRNA selected by the client or not.
	 */
	private boolean						selected				 = false;

	/**
	 * List of targets for the MiRNA.
	 */
	private List<TargetView>	targetRows			 = new ArrayList<>();

	/***
	 * Constructor that initialize the data with information from the
	 * parameter*object.
	 ** 
	 * @param miRna
	 *          original MiRNA from which the data will be initialized
	 */
	protected MiRNAView(MiRNA miRna) {
		// MiRNA is not db entity so don't pass it to super constructor
		super(null);
	}

	/**
	 * Default constructor. Should be used only for deserialization.
	 */
	protected MiRNAView() {
	}

	/**
	 * Default constructor that creates a copy.
	 * 
	 * @param original
	 *          origianl object that should be copied
	 */
	public MiRNAView(MiRNAView original) {
		super(null);
		name = original.getName();
		selected = original.isSelected();
		targetRows.addAll(original.getTargetRows());
	}

	/**
	 * Comparator of the objects by their name.
	 * 
	 * @author Piotr Gawron
	 * 
	 */
	public static class NameComparator implements Comparator<MiRNAView> {
		/**
		 * Default string comparator.
		 */
		private StringComparator stringComparator = new StringComparator();

		@Override
		public int compare(MiRNAView arg0, MiRNAView arg1) {
			return stringComparator.compare(arg0.getName(), arg1.getName());
		}

	}

	/**
	 * @return the selected
	 */
	public boolean isSelected() {
		return selected;
	}

	/**
	 * @param selected
	 *          the selected to set
	 */
	public void setSelected(boolean selected) {
		this.selected = selected;
	}

	/**
	 * @return the targetRows
	 */
	public List<TargetView> getTargetRows() {
		return targetRows;
	}

	/**
	 * @param targetRows
	 *          the targetRows to set
	 */
	public void setTargetRows(List<TargetView> targetRows) {
		this.targetRows = targetRows;
	}

	/**
	 * @return the mirBase id
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *          mirBase id
	 */
	public void setName(String name) {
		this.name = name;
	}

	@Override
	public String getUniqueId() {
		return name;
	}

	@Override
	public void setUniqueId(String uniqueId) {
		this.name = uniqueId;
	}

}
